const webpack = require('webpack');
const { resolve } = require('path');
const ProgressPlugin = require('webpack/lib/ProgressPlugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');

const { NoEmitOnErrorsPlugin, NamedModulesPlugin } = require('webpack');

const config = {
  entry: {
    mode: resolve('src', 'mode', 'kevscript.js'),
    lint: resolve('src', 'lint', 'lint.js'),
    hint: resolve('src', 'hint', 'hint.js'),
  },
  output: {
    filename: '[name].js',
    path: resolve('dist'),
    libraryTarget: 'umd'
  },
  externals: {
    'codemirror': 'CodeMirror',
    'kevoree-registry-client': 'KevoreeRegistryClient',
    'tiny-conf': 'TinyConf',
    'kevoree-library': 'KevoreeLibrary',
    'kevoree-kevscript': 'KevoreeKevscript'
  },
  plugins: [
    new NoEmitOnErrorsPlugin(),
    new ProgressPlugin(),
    new NamedModulesPlugin({})
  ],
};

if (process.env.NODE_ENV !== 'production') {
  config.entry.test = resolve('test', 'main.js');
  config.plugins.push(
    new webpack.HotModuleReplacementPlugin(),
    new HtmlWebpackPlugin({
      "template": "./test/index.html",
      "filename": "./index.html",
      "inject": false,
      "compile": true,
      "favicon": false,
      "showErrors": true,
      "xhtml": true,
    }));
  config.devtool = 'source-map';
}

module.exports = config;
